@@ -1,3 +1,4 @@
+#!/usr/bin/env python3
 """
 @author: RealStickman
 """
@@ -10,34 +11,14 @@ from pathlib import Path
 from tkinter import messagebox, filedialog, ttk
 import praw
 import prawcore
+from multiprocessing import Process
 
-#Reads the config and places the values into the global namespace
-def readconf():
-    print("Reading Config file")
-    config.read('grab.ini')
-    global lim
-    lim = int(config['CONFIG']['limit'])
-    global category
-    category = config['CONFIG']['category']
-    global subl
-    subl = config['CONFIG']['subs']
-    global sublist
-    sublist = subl.replace('.empty.', '')
-    global path
-    path = config['CONFIG']['path']
-    global seltheme
-    seltheme = config['CONFIG']['theme']
+# variables
+import progvars
+# modules
+import modules
 
-#Write the configuration file
-def writeconf():
-    print("placeholder")
-    config['CONFIG'] = {'limit': lim,
-                        'category': category,
-                        'subs': sublist,
-                        'path': path,
-                        'theme': seltheme}
-    with open('grab.ini', 'w') as configfile:
-        config.write(configfile)
+import grab
 
 def addfr():
     #remove add/remove subreddit buttons
@@ -93,18 +74,16 @@ def save():
             subreddit.title
 
             #read the config
-            readconf()
-            # explicitly assign sublist variable, otherwise it complains about the variable being called before assignment
-            global sublist
+            modules.readconf()
             #add entered subreddit
-            sublist += subreddit.display_name + ";"
+            progvars.sublist += subreddit.display_name + ";"
             #that's were we're gonna put it
-            writeconf()
+            modules.writeconf()
             print("Added " + subreddit.display_name)
 
             #reformatting list of subs
-            lsublist = sublist.split(";")
-            labelsublist = [x for x in lsublist if x]
+            progvars.sublf = progvars.sublist.split(";")
+            labelsublist = [x for x in progvars.sublf if x]
             # displays list of subreddits
             lbox.delete(0, 'end')
             for i in range(len(labelsublist)):
@@ -134,37 +113,46 @@ def stop():
     sys.exit()
     exit()
 
-#runs the program
-def run():
+# showing info message and have it be interactable
+def runprog():
+    print("works runprog")
     messagebox.showinfo("Running", "Program is running")
-    #python3 was used to deal with Debian still linking python with python2 instead of python3
-    try:
-        child = subprocess.Popen('grab-reddit', shell=True, stdout=subprocess.PIPE, text=True)
-    except:
-        child = subprocess.Popen('python3 grab.py', shell=True, stdout=subprocess.PIPE, text=True)
+    main.update_idletasks()
 
-    data = child.communicate()[0]
-    rc = child.returncode
-    print(data)
-    print("grab.py finished with exit code " + str(rc))
-    if rc == 0:
-        messagebox.showinfo("Success", "Program ran without errors")
-    else:
-        messagebox.showerror("Error", "Error code " + str(rc))
-        with open("log.txt", "w") as log:
-            log.write(data)
-            log.write("\nError code " + str(rc))
+# process that executes the download
+def rundl():
+    print("works rundl")
+    # to make sure the number of subs is correct
+    modules.readconf()
+    # download images from reddit
+    modules.multiprocdl()
+
+# runs the program
+def run():
+    # set up processes
+    procprog = Process(target = runprog())
+    procrundl = Process(target = rundl())
+
+    # start processes
+    procrundl.start()
+    procprog.start()
+
+    # make sure both processes are finished
+    procprog.join()
+    procrundl.join()
+
+    messagebox.showinfo("Finished", "Program has finished")
 
 def listbox():
     result = tk.messagebox.askyesno("Delete", "Remove selected subreddit from list?", icon='warning')
     if result == True:
         #read the config
-        readconf()
+        modules.readconf()
         selec = lbox.curselection()
         val = lbox.get(selec)
         global sublist
-        sublist = sublist.replace(";" + val, '')
-        writeconf()
+        progvars.sublist = progvars.sublist.replace(";" + val, '')
+        modules.writeconf()
         print("Deleted " + val)
         updatlist()
     else:
@@ -172,58 +160,57 @@ def listbox():
 
 def updatlist():
     #read the config
-    readconf()
-    lcusubl = sublist.split(";")
+    modules.readconf()
+    lcusubl = progvars.sublist.split(";")
     labsubl = [x for x in lcusubl if x]
     lbox.delete(0, 'end')
     for i in range(len(labsubl)):
         lbox.insert(0, labsubl[i])
 
 def catsel():
-    readconf()
-    global category
+    modules.readconf()
     #sets category based on the radiobutton selected
     if catvar.get() == 1:
-        category = "controversial"
-        writeconf()
+        progvars.category = "controversial"
+        modules.writeconf()
         print("Setting category to controversial")
     # elif catvar.get() == 2:
-    #     category = "gilded"
+    #     progvars.category = "gilded"
     #     writeconf()
     #     print("Setting category to gilded")
     elif catvar.get() == 3:
-        category = "hot"
-        writeconf()
+        progvars.category = "hot"
+        modules.writeconf()
         print("Setting category to hot")
     elif catvar.get() == 4:
-        category = "new"
-        writeconf()
+        progvars.category = "new"
+        modules.writeconf()
         print("Setting category to new")
     elif catvar.get() == 5:
-        category = "rising"
-        writeconf()
+        progvars.category = "rising"
+        modules.writeconf()
         print("Setting category to rising")
     elif catvar.get() == 6:
-        category = "top"
-        writeconf()
+        progvars.category = "top"
+        modules.writeconf()
         print("Setting category to top")
     else:
         print("Something went wrong")
 
 #save the limit
 def savelim():
-    readconf()
+    modules.readconf()
     print("New limit: " + limspin.get())
     global lim
-    lim = limspin.get()
-    writeconf()
+    progvars.lim = limspin.get()
+    modules.writeconf()
 
 #expands the window to show all settings
 def winexp():
     global extend
     extend = ttk.Frame(main)
     extend.grid(row=0, column=2, sticky="W")
-    readconf()
+    modules.readconf()
 
     #destroy button to expand window
     butexp.destroy()
@@ -244,7 +231,7 @@ def winexp():
 
     # Path
     global pathlab
-    pathlab = ttk.Label(varfr, text="Path: " + path)
+    pathlab = ttk.Label(varfr, text="Path: " + progvars.path)
     pathlab.grid(row=0, column=1, sticky="W")
     global butpath
     butpath = ttk.Button(varfr, text="...", command=dirdialog)
@@ -256,7 +243,7 @@ def winexp():
     limspin.grid(row=1, column=1, sticky="W")
     # show currently set limit
     i = 0
-    while i < lim:
+    while i < progvars.lim:
         limspin.invoke(element="buttonup")
         i += 1
 
@@ -304,23 +291,23 @@ def winexp():
     cattop.grid(row=1, column=1, sticky="W")
 
     # selects appropriate button to show which one is active
-    if category == 'controversial':
-        print("Current selected category is " + category)
+    if progvars.category == 'controversial':
+        print("Current selected category is " + progvars.category)
         catcont.select()
-    # elif category == 'gilded':
-    #     print("Current selected category is " + category)
+    # elif progvars.category == 'gilded':
+    #     print("Current selected category is " + progvars.category)
     #     catgil.select()
-    elif category == 'hot':
-        print("Current selected category is " + category)
+    elif progvars.category == 'hot':
+        print("Current selected category is " + progvars.category)
         cathot.select()
-    elif category == 'new':
-        print("Current selected category is " + category)
+    elif progvars.category == 'new':
+        print("Current selected category is " + progvars.category)
         catnew.select()
-    elif category == 'rising':
-        print("Current selected category is " + category)
+    elif progvars.category == 'rising':
+        print("Current selected category is " + progvars.category)
         catris.select()
-    elif category == 'top':
-        print("Current selected category is " + category)
+    elif progvars.category == 'top':
+        print("Current selected category is " + progvars.category)
         cattop.select()
 
     #close expanded view
@@ -330,29 +317,26 @@ def winexp():
 
 #setlight and setdark could maybe be integrated into one function.
 def setlight():
-    readconf()
-    global seltheme
-    if seltheme != "light":
-        seltheme = "light"
-        print("Setting " + seltheme + " theme")
+    modules.readconf()
+    if progvars.seltheme != "light":
+        progvars.seltheme = "light"
+        print("Setting " + progvars.seltheme + " theme")
         changetheme()
     else:
-        print("Program is already using the " + seltheme + " theme")
+        print("Program is already using the " + progvars.seltheme + " theme")
 
 def setdark():
-    readconf()
-    global seltheme
-    if seltheme != "dark":
-        seltheme = "dark"
-        print("Setting " + seltheme + " theme")
+    modules.readconf()
+    if progvars.seltheme != "dark":
+        progvars.seltheme = "dark"
+        print("Setting " + progvars.seltheme + " theme")
         changetheme()
     else:
-        print("Program is already using the " + seltheme + " theme")
+        print("Program is already using the " + progvars.seltheme + " theme")
 
 #code pulled out of setlight and setdark in order to avoid huge blocks of duplicate code
 def changetheme():
-    print("placeholder")
-    writeconf()
+    modules.writeconf()
     root.destroy()
     try:
         subprocess.Popen('grab-reddit-gui', shell=True, text=True)
@@ -369,14 +353,13 @@ def winshr():
     butexp.grid(row=1, column=1, sticky="W")
 
 def dirdialog():
-    readconf()
+    modules.readconf()
     #explicitly assign path variable, otherwise it complains about the variable being called before assignment
-    global path
-    path = tk.filedialog.askdirectory(parent=root, initialdir=path, title="Download directory")
-    writeconf()
+    progvars.path = tk.filedialog.askdirectory(parent=root, initialdir=progvars.path, title="Download directory")
+    modules.writeconf()
     global pathlab
     pathlab.destroy()
-    pathlab = ttk.Label(varfr, text="Path: " + path)
+    pathlab = ttk.Label(varfr, text="Path: " + progvars.path)
     pathlab.grid(row=0, column=1, sticky="W")
 
 def main():
@@ -388,31 +371,27 @@ def main():
     #choose default path
     pathdef = str(os.path.join(Path.home(), "Downloads", "grab-bot"))
 
-    global config
-    #config initialisation
-    config = configparser.ConfigParser()
-
     #make sure the config file is present and correct
     try:
-        config.read('grab.ini')
-        lim = config['CONFIG']['limit']
-        category = config['CONFIG']['category']
-        config['CONFIG']['subs']
-        path = config['CONFIG']['path']
-        seltheme = config['CONFIG']['theme']
+        progvars.config.read('grab.ini')
+        progvars.lim = progvars.config['CONFIG']['limit']
+        progvars.category = progvars.config['CONFIG']['category']
+        progvars.config['CONFIG']['subs']
+        progvars.path = progvars.config['CONFIG']['path']
+        progvars.seltheme = progvars.config['CONFIG']['theme']
         print("Config is present and correct")
     except KeyError:
-        config['CONFIG'] = {'limit': '10',
+        progvars.config['CONFIG'] = {'limit': '10',
                             'category': 'hot',
                             'subs': '.empty.;',
                             'path': pathdef,
                             'theme': 'light'}
         with open('grab.ini', 'w') as configfile:
-            config.write(configfile)
+            progvars.config.write(configfile)
         print("Created config or fixed missing options")
 
     #read config
-    readconf()
+    modules.readconf()
 
     global root
     #main window
@@ -425,8 +404,8 @@ def main():
 
     global background
     global foreground
-    print("Using " + seltheme + " theme")
-    if seltheme == "light":
+    print("Using " + progvars.seltheme + " theme")
+    if progvars.seltheme == "light":
         root.configure(bg="#ffffff")
         #light theme
         theme.theme_create('light')
@@ -465,7 +444,7 @@ def main():
         #use light theme
         theme.theme_use('light')
 
-    elif seltheme == "dark":
+    elif progvars.seltheme == "dark":
         root.configure(bg="#000000")
         #dark theme
         theme.theme_create('dark')
@@ -524,12 +503,11 @@ def main():
     var = tk.StringVar()
 
     global cnnsublist
-    global lsublist
     global labelsublist
     #change the list of subs to fit
-    cnnsublist = sublist.replace(';', '\n')
-    lsublist = sublist.split(";")
-    labelsublist = [x for x in lsublist if x]
+    cnnsublist = progvars.sublist.replace(';', '\n')
+    progvars.sublf = progvars.sublist.split(";")
+    labelsublist = [x for x in progvars.sublf if x]
 
     global lbox
     #multiple doesn't work with my method
